package json

import scala.util.parsing.combinator._
import java.io.FileReader

/**
 * A parser for JSON data with conversion to xml.
 * From "Programming in Scala", chapter 31.
 */
object JSON2XML extends JavaTokenParsers {

  val tab: scala.runtime.RichString = "  "
  
  //----------------------------------------------------------------------------
  // Parser Definitions
  //----------------------------------------------------------------------------
  
  // an object is a map of members
  def obj: Parser[Map[String, Any]] = "{"~> repsep(member, ",") <~"}" ^^ (Map() ++ _)
    
  // an array is a square-bracketed, comma-separated list
  def arr: Parser[List[Any]] = "["~> repsep(value, ",") <~"]"
  
  // a member is a (name,value) pair
  def member: Parser[(String, Any)] = stringLiteral~":"~value ^^ { case name~":"~value => (name, value) }
  
  // a value is an object, an array, or a primitive/literal
  def value: Parser[Any] = (
      obj
    | arr
    | stringLiteral
    | floatingPointNumber ^^ (_.toDouble)
    | "null" ^^ (x => null)
    | "true" ^^ (x => true)
    | "false" ^^ (x => false)
  )
  
  /*----------------------------------------------------------------------------
   * Main
   *--------------------------------------------------------------------------*/
  def main(args: Array[String]) = {
    if (args.length == 0) {
      println("syntax: scala json2xml.JSON2XML <inputfile>")
      exit(1)
    }
    val reader = new FileReader(args(0))
    println("parsing " + args(0))
    println
    val res = parseAll(value, reader)
    println("parse result: " + res)
    println
    println("converting into xml")
    println
    printXML(res.get)
    println
  }
  
  def printXML(obj: Any): Unit = printXML(obj, 0)
  
  def printXML(obj: Any, level: Int): Unit = {    
    // match object type: Map or List
    obj match {
      case p: Map[_, _] =>
        for((k,v) <- p) {
          val k1 = fix(k)
          print(tabs(level) + "<" + k1 + ">")
          // match value type
          var primitive = true
          v match {
            case s: String => print(fix(s))
            case i: Int => print(i)
            case d: Double => print(d)
            case _ => {
              println()
              printXML(v, level + 1)
              primitive = false
            }
          }
          if (!primitive)
            print(tabs(level))
          println("</" + k1 + ">")
        }        
      case t: List[_] =>
        for (i <- 0 to t.length - 1)
          printf(tabs(level) + """<item idx="%d">%s</item>%n""", i, fix(t(i)))      
      case _ =>
        throw new RuntimeException("no match for " + obj)
    }
  }
  
  /**
   * Remove quotes
   */
  def fix(t: Any) =
    t match {
      case s: String => s.substring(1, s.length - 1)
      case _ => t  
    }
    
  /**
   * Repeat n tabs
   */
  def tabs(n: Int): String = {
      tab * n
  }
}
