package statechart

import scala.util.parsing.combinator.syntactical.StandardTokenParsers
import scala.collection.mutable.Map

/**
 * Combinator parser for statecharts with conversion in Scala object representation.
 */
object StatechartParser extends StandardTokenParsers {

  def statechart: Parser[Statechart] = "statechart" ~> ident ~ ":" ~ properties ~ initialState ~ states ^^ {
    case name ~ ":" ~ properties ~ initialState ~ states => new Statechart(name, properties, initialState, states)
  }
  
  def state: Parser[State] = "state" ~> ident ~ ":" ~ opt(onEntry) ~ opt(onExit) ~ transitions ^^ {
    case ident ~ ":" ~ entry ~ exit ~ transitions => { new State(ident, entry, exit, transitions) }
  }
  
  def extendedState: Parser[ExtendedState] = "extendedState" ~> ident ~ ":" ~ substates ~ initialState ~ transitions ^^ {
    case id ~ ":" ~ substates ~ initialState ~ transitions => new ExtendedState(id, substates, initialState, transitions)
  }
  
  def transition: Parser[Transition] = ident ~ opt(guard) ~ "->" ~ ident ^^ {
    case action ~ guard ~ "->" ~ nextState => new Transition(action, guard, nextState)
  }
  
  def properties: Parser[Map[String,Any]] = "properties" ~> ":" ~> repsep(property, ",") ^^ (Map() ++ _)
  
  def property: Parser[(String,Any)] = ident ~ "=" ~ initialValue ^^ { case key ~ "=" ~ value => (key, value) }
  
  def substates: Parser[List[String]] = "substates" ~> ":" ~> repsep(ident, ",")
  
  def initialValue: Parser[Any] = ( numericLit ^^ (x => x.toInt) | "true" ^^ (x => true) | "false" ^^ (x => false))  
  
  def transitions: Parser[List[Transition]] = "transitions" ~> ":" ~> rep(transition)  
  
  def guard: Parser[Condition] = "[" ~> condition <~ "]"
  
  def condition: Parser[Condition] = opt("not") ~ ident ~ opt(comparison) ^^ {
    case not ~ ident ~ comp => new Condition(not, ident, comp)         
  }
  
  def comparison: Parser[Comparison] = symbol ~ numericLit ^^ { case symbol ~ numericLit => new Comparison(symbol, numericLit) }
  
  def symbol: Parser[String] = "gt" | "lt" | "eq"
  
  def states: Parser[List[State]] = rep1(state | extendedState)  
  
  def initialState: Parser[String] = "initialState" ~> ":" ~> ident
  
  def onEntry: Parser[List[String]] = "onEntry" ~> ":" ~> repsep(ident, ",")
  
  def onExit: Parser[List[String]] = "onExit" ~> ":" ~> repsep(ident, ",")
  
  // Hilfsfunktion
  def scan(s: String) = new StatechartParser.lexical.Scanner(s)
  
  // Fuer den Lexer benoetigt
  lexical.delimiters ++= List("[", "]", ",", "(", ")", ":", "=", "->")
  
  lexical.reserved ++= List("statechart", "properties", "extendedState",
      "substates", "transitions", "onEntry", "onExit", "not", "gt", "lt",
      "eq", "state", "initialState", "true", "false")  
}
